/*
 * Copyright (c) 2015 Samsung Electronics Co., Ltd All Rights Reserved
 *
 * Licensed under the Apache License, Version 2.0 (the License);
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 * http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an AS IS BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

#include <Elementary.h>
#include <media_content.h>
#include <app_debug.h>
#include <app_media.h>

#include "data/datamgr.h"

struct _type_info {
	struct data_ops *(*func_get_ops)(void);
};

static struct _type_info g_type_info[E_DATA_MAX] = {
	[E_DATA_MEDIA] = {
		.func_get_ops = mediadata_get_ops
	},
	[E_DATA_ALBUM] = {
		.func_get_ops = albumdata_get_ops
	},
	[E_DATA_FOLDER] = {
		.func_get_ops = folderdata_get_ops
	},
};

struct datamgr *datamgr_create(int data_type, const char *media_type,
			int source_type)
{
	struct datamgr *dmgr;
	struct data_ops *ops;
	void *handle;

	if (!media_type || data_type < 0 || data_type >= E_DATA_MAX ||
		source_type < 0 || source_type >= E_SOURCE_MAX) {
		_ERR("invalid argument");
		return NULL;
	}

	dmgr = calloc(1, sizeof(*dmgr));
	if (!dmgr) {
		_ERR("failed to allocate datamgr");
		return NULL;
	}

	ops = g_type_info[data_type].func_get_ops();
	if (!ops) {
		_ERR("failed to get data operations");
		free(dmgr);
		return NULL;
	}

	if (!ops->create || !ops->destroy || !ops->get_list ||
		!ops->get_count || !ops->get_group || !ops->free_group) {
		_ERR("invalid data operations");
		free(dmgr);
		return NULL;
	}

	handle = ops->create(media_type, source_type);
	if (!handle) {
		_ERR("failed to create data handle");
		free(dmgr);
		return NULL;
	}

	dmgr->ops = ops;
	dmgr->handle = handle;

	return dmgr;
}

void datamgr_destroy(struct datamgr *dmgr)
{
	if (!dmgr) {
		_ERR("failed to get datamgr");
		return;
	}

	dmgr->ops->destroy(dmgr->handle);
	free(dmgr);
}
